@@ -1,5 +1,25 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
 """FreeSurfer tools interfaces."""
 
 import os.path as op
@@ -7,10 +27,6 @@ from pathlib import Path
 import nibabel as nb
 import numpy as np
 
-from skimage import morphology as sim
-from scipy.ndimage.morphology import binary_fill_holes
-from nilearn.image import resample_to_img, new_img_like
-
 from nipype.utils.filemanip import copyfile, filename_to_list, fname_presuffix
 from nipype.interfaces.base import (
     isdefined,
@@ -25,7 +41,7 @@ from nipype.interfaces import freesurfer as fs
 from nipype.interfaces.base import SimpleInterface
 from nipype.interfaces.freesurfer.preprocess import ConcatenateLTA, RobustRegister
 from nipype.interfaces.freesurfer.utils import LTAConvert
-from .registration import BBRegisterRPT, MRICoregRPT
+from .reportlets.registration import BBRegisterRPT, MRICoregRPT
 
 
 class StructuralReference(fs.RobustTemplate):
@@ -183,12 +199,12 @@ class FSDetectInputs(SimpleInterface):
             hires_enabled=self.inputs.hires_enabled,
         )
 
-        if t2w is not None:
-            self._results["use_t2w"] = True
+        self._results["use_t2w"] = t2w is not None
+        if self._results["use_t2w"]:
             self._results["t2w"] = t2w
 
-        elif flair is not None:
-            self._results["use_flair"] = True
+        self._results["use_flair"] = flair is not None
+        if self._results["use_flair"]:
             self._results["flair"] = flair
 
         if self._results["hires"]:
@@ -238,7 +254,7 @@ class PatchedConcatenateLTA(TruncateLTA, ConcatenateLTA):
     in FreeSurfer, that was
     `fixed here <https://github.com/freesurfer/freesurfer/pull/180>`__.
 
-    The original FMRIPREP's issue is found
+    The original fMRIPrep's issue is found
     `here <https://github.com/nipreps/fmriprep/issues/768>`__.
 
     the fix is now done through mixin with TruncateLTA
@@ -329,9 +345,7 @@ class _MedialNaNsOutputSpec(TraitedSpec):
 
 
 class MedialNaNs(SimpleInterface):
-    """
-    The MedialNaNs converts from arbitrary units to rad/s
-    """
+    """Convert values on medial wall to NaNs."""
 
     input_spec = _MedialNaNsInputSpec
     output_spec = _MedialNaNsOutputSpec
@@ -355,6 +369,7 @@ def fix_lta_length(lta_file):
     Examples
     --------
     No changes are made to a valid transform file:
+
     >>> valid_transform = Path(test_data) / 'valid_transform.lta'
     >>> orig_contents = valid_transform.read_text()
     >>> fix_lta_length(valid_transform)
@@ -363,6 +378,7 @@ def fix_lta_length(lta_file):
     True
 
     Invalid transform files are files with > 255 characters in any line:
+
     >>> invalid_transform = Path(test_data) / 'long_path_transform.lta'
     >>> orig_contents = invalid_transform.read_text()
     >>> any(len(line) > 255 for line in orig_contents.splitlines(keepends=True))
@@ -393,6 +409,8 @@ def fix_lta_length(lta_file):
 
 
 def inject_skullstripped(subjects_dir, subject_id, skullstripped):
+    from nilearn.image import resample_to_img, new_img_like
+
     mridir = op.join(subjects_dir, subject_id, "mri")
     t1 = op.join(mridir, "T1.mgz")
     bm_auto = op.join(mridir, "brainmask.auto.mgz")
@@ -454,6 +472,9 @@ def refine_aseg(aseg, ball_size=4):
          cerebral brain is segmented out).
 
     """
+    from skimage import morphology as sim
+    from scipy.ndimage.morphology import binary_fill_holes
+
     # Read aseg data
     bmask = aseg.copy()
     bmask[bmask > 0] = 1
@@ -476,6 +497,8 @@ def grow_mask(anat, aseg, ants_segs=None, ww=7, zval=2.0, bw=4):
     https://github.com/nipy/mindboggle/blob/master/mindboggle/guts/segment.py#L1660
 
     """
+    from skimage import morphology as sim
+
     selem = sim.ball(bw)
 
     if ants_segs is None:
